from six import string_types, text_type, binary_type


def get_query_params(request):
    try:
        return request.query_params
    except:
        return request.QUERY_PARAMS


def get_request_data(request):
    try:
        return request.data
    except:
        return request.DATA


def is_str(value):
    return isinstance(value, (string_types, text_type, binary_type))


def is_valid_digit(value):
    if isinstance(value, (int, float)):
        return True
    elif is_str(value):
        return value.isdigit()
    return False



from django.dispatch import receiver
from django.db.models.signals import m2m_changed, post_save

from bulbs.content.models import Content

from .models import ReportContent, Contribution, ContributorRole
from .utils import update_content_contributions


@receiver(m2m_changed, sender=Content.authors.through)
def update_contributions(sender, instance, action, model, pk_set, **kwargs):
    """Creates a contribution for each author added to an article.
    """
    # action = kwargs.get('action')
    # import pdb; pdb.set_trace()
    if action != 'pre_add':
        return
    else:
        # model = kwargs.get('model')
        # pk_set = kwargs.get('pk_set')
        for author in model.objects.filter(pk__in=pk_set):
            update_content_contributions(instance, author)


@receiver(post_save, sender=Content)
def index_reportcontent(sender, instance, **kwargs):
    """
    Indexes the reporting document for the piece of content
    """
    try:
        proxy = ReportContent.reference.get(id=instance.id)
        proxy.index()
    except:
        pass


@receiver(post_save, sender=Contribution)
def index_relations(sender, instance, **kwargs):
    """
    We need to update the ReportContent object whenver a contribution is added.
    """
    try:
        proxy = ReportContent.reference.get(id=instance.content_id)
        proxy.index()
    except:
        pass


@receiver(post_save, sender=ContributorRole)
def update_rates(sender, instance, **kwargs):
    for contribution in instance.contribution_set.all():
        contribution.save()


# @receiver(post_save, sender=ContributionOverride)
# def update_contribution_from_override(sender, instance, **kwargs):




from rest_framework import filters

from bulbs.utils.methods import get_query_params

class CaseInsensitiveBooleanFilter(filters.BaseFilterBackend):
    """Set a boolean_fields tuple on the viewset and set this class as a
    filter_backend to filter listed fields through a case-insensitive transformation
    to be used for filtering. i.e. query params such as 'true' become boolean
    True, and params with a value 'false' become boolean False."""

    def filter_queryset(self, request, queryset, view):

        boolean_fields = getattr(view, 'boolean_fields', None)

        if not boolean_fields:
            return queryset

        boolean_filters = {}
        for field in boolean_fields:
            if field in get_query_params(request):
                val = get_query_params(request)[field].lower()
                if val == 'true':
                    boolean_filters[field] = True
                elif val == 'false':
                    boolean_filters[field] = False

        if len(boolean_filters) > 0:
            return queryset.filter(**boolean_filters)

        return queryset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('content', '0003_auto_20150513_2326'),
        ('contributions', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseOverride',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('rate', models.IntegerField(default=0)),
                ('updated_on', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-updated_on',),
            },
        ),
        migrations.CreateModel(
            name='FreelanceProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_freelance', models.BooleanField(default=True)),
                ('is_manager', models.BooleanField(default=True)),
                ('payment_date', models.DateTimeField(null=True, blank=True)),
                ('contributor', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='LineItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('amount', models.IntegerField(default=0)),
                ('note', models.TextField()),
                ('payment_date', models.DateTimeField(null=True, blank=True)),
                ('contributor', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='OverrideProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('contributor', models.ForeignKey(related_name='overrides', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Rate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.IntegerField(null=True, choices=[(0, b'Flat Rate'), (1, b'FeatureType'), (2, b'Hourly'), (3, b'Manual'), (4, b'Override')])),
                ('updated_on', models.DateTimeField(auto_now=True)),
                ('rate', models.IntegerField()),
            ],
            options={
                'ordering': ('-updated_on',),
            },
        ),
        migrations.AddField(
            model_name='contribution',
            name='force_payment',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='contribution',
            name='minutes_worked',
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name='contribution',
            name='payment_date',
            field=models.DateTimeField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='contributorrole',
            name='payment_type',
            field=models.IntegerField(default=3, choices=[(0, b'Flat Rate'), (1, b'FeatureType'), (2, b'Hourly'), (3, b'Manual')]),
        ),
        migrations.AlterField(
            model_name='contribution',
            name='content',
            field=models.ForeignKey(related_name='contributions', to='content.Content'),
        ),
        migrations.AlterField(
            model_name='contribution',
            name='contributor',
            field=models.ForeignKey(related_name='contributions', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='ContributionOverride',
            fields=[
                ('baseoverride_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.BaseOverride')),
                ('contribution', models.ForeignKey(related_name='override_contribution', to='contributions.Contribution')),
            ],
            bases=('contributions.baseoverride',),
        ),
        migrations.CreateModel(
            name='FeatureTypeOverride',
            fields=[
                ('baseoverride_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.BaseOverride')),
                ('feature_type', models.ForeignKey(to='content.FeatureType')),
            ],
            bases=('contributions.baseoverride',),
        ),
        migrations.CreateModel(
            name='FeatureTypeRate',
            fields=[
                ('rate_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.Rate')),
                ('feature_type', models.ForeignKey(related_name='feature_type_rates', to='content.FeatureType')),
                ('role', models.ForeignKey(related_name='feature_type_rates', to='contributions.ContributorRole', null=True)),
            ],
            bases=('contributions.rate',),
        ),
        migrations.CreateModel(
            name='FlatRate',
            fields=[
                ('rate_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.Rate')),
                ('role', models.ForeignKey(related_name='flat_rates', to='contributions.ContributorRole')),
            ],
            bases=('contributions.rate',),
        ),
        migrations.CreateModel(
            name='FlatRateOverride',
            fields=[
                ('baseoverride_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.BaseOverride')),
            ],
            bases=('contributions.baseoverride',),
        ),
        migrations.CreateModel(
            name='HourlyOverride',
            fields=[
                ('baseoverride_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.BaseOverride')),
            ],
            bases=('contributions.baseoverride',),
        ),
        migrations.CreateModel(
            name='HourlyRate',
            fields=[
                ('rate_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.Rate')),
                ('role', models.ForeignKey(related_name='hourly_rates', to='contributions.ContributorRole')),
            ],
            bases=('contributions.rate',),
        ),
        migrations.CreateModel(
            name='ManualRate',
            fields=[
                ('rate_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.Rate')),
                ('contribution', models.ForeignKey(related_name='manual_rates', to='contributions.Contribution')),
            ],
            bases=('contributions.rate',),
        ),
        migrations.AddField(
            model_name='overrideprofile',
            name='role',
            field=models.ForeignKey(related_name='overrides', to='contributions.ContributorRole'),
        ),
        migrations.AlterUniqueTogether(
            name='overrideprofile',
            unique_together=set([('contributor', 'role')]),
        ),
        migrations.AddField(
            model_name='hourlyoverride',
            name='profile',
            field=models.ForeignKey(related_name='override_hourly', to='contributions.OverrideProfile'),
        ),
        migrations.AddField(
            model_name='flatrateoverride',
            name='profile',
            field=models.ForeignKey(related_name='override_flatrate', to='contributions.OverrideProfile'),
        ),
        migrations.AddField(
            model_name='featuretypeoverride',
            name='profile',
            field=models.ForeignKey(related_name='override_feature_type', to='contributions.OverrideProfile'),
        ),
        migrations.AlterUniqueTogether(
            name='featuretyperate',
            unique_together=set([('role', 'feature_type')]),
        ),
    ]



import json
import logging
import random

from elasticsearch_dsl.connections import connections

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.test import TestCase
from djes.apps import indexable_registry
from djes.management.commands.sync_es import get_indexes, sync_index

from model_mommy import mommy
from rest_framework.test import APIClient
from six import PY3

from bulbs.content.models import Content


def make_content(*args, **kwargs):
    if "make_m2m" not in kwargs:
        kwargs["make_m2m"] = True

    if len(args) == 1:
        klass = args[0]
    else:
        models = indexable_registry.families[Content]
        model_keys = [key for key in models.keys() if key != "content_content"]
        key = random.choice(model_keys)
        klass = indexable_registry.all_models[key]

    content = mommy.make(klass, **kwargs)
    return content


class JsonEncoder(json.JSONEncoder):
    def default(self, value):
        """Convert more Python data types to ES-understandable JSON."""
        iso = _iso_datetime(value)
        if iso:
            return iso
        if not PY3 and isinstance(value, str):
            return unicode(value, errors='replace')  # TODO: Be stricter.
        if isinstance(value, set):
            return list(value)
        return super(JsonEncoder, self).default(value)


def _iso_datetime(value):
    """
    If value appears to be something datetime-like, return it in ISO format.

    Otherwise, return None.
    """
    if hasattr(value, 'strftime'):
        if hasattr(value, 'hour'):
            return value.isoformat()
        else:
            return '%sT00:00:00' % value.isoformat()


class BaseIndexableTestCase(TestCase):
    """A TestCase which handles setup and teardown of elasticsearch indexes."""

    elasticsearchLogger = logging.getLogger('elasticsearch')

    def setUp(self):
        """ If you're reading this. I am gone and dead, presumably.
            Elasticsearch's logging is quite loud and lets us know
            about anticipated errors, so I set the level to ERROR only.
            If elasticsearch is giving you trouble in tests and you
            aren't seeing any info, get rid of this. God bless you.
        """
        self.elasticsearchLogger.setLevel(logging.ERROR)
        self.es = connections.get_connection("default")
        self.indexes = get_indexes()

        for index in list(self.indexes):
            self.es.indices.delete_alias("{}*".format(index), "_all", ignore=[404])
            self.es.indices.delete("{}*".format(index), ignore=[404])

        for index, body in self.indexes.items():
            sync_index(index, body)

    def tearDown(self):
        for index in list(self.indexes):
            self.es.indices.delete_alias("{}*".format(index), "_all", ignore=[404])
            self.es.indices.delete("{}*".format(index), ignore=[404])


class BaseAPITestCase(BaseIndexableTestCase):
    """A base test case, allowing tearDown and setUp of the ES index"""

    def setUp(self):
        super(BaseAPITestCase, self).setUp()
        User = get_user_model()
        admin = self.admin = User.objects.create_user("admin", "tech@theonion.com", "secret")
        admin.is_staff = True
        admin.save()
        self.api_client = APIClient()
        self.api_client.force_authenticate(user=admin)
        # reverse("content-detail")

    def give_permissions(self):
        publish_perm = Permission.objects.get(codename="publish_content")
        change_perm = Permission.objects.get(codename="change_content")
        promote_perm = Permission.objects.get(codename="promote_content")
        self.admin.user_permissions.add(publish_perm, change_perm, promote_perm)

    def give_author_permissions(self):
        publish_perm = Permission.objects.get(codename="publish_own_content")
        self.admin.user_permissions.add(publish_perm)

    def remove_permissions(self):
        self.admin.user_permissions.clear()
